<?php

namespace Quivers\Integration\Block\Sales\Adminhtml\Order;
use Quivers\Integration\LogTracking;
class Tax extends \Magento\Framework\View\Element\Template
{
    /**
     * Tax configuration model
     *
     * @var \Magento\Tax\Model\Config
     */
    protected $_config;

    /**
     * @var Order
     */
    protected $_order;

    /**
     * @var \Magento\Framework\DataObject
     */
    protected $_source;
    protected $_scopeConfig;
    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Tax\Model\Config $taxConfig
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Tax\Model\Config $taxConfig,
        array $data = []
    ) {
        $this->_config = $taxConfig;
        $this->_scopeConfig = $scopeConfig;
        $this->_checkoutSession = $checkoutSession;
        parent::__construct($context, $data);
    }

    /**
     * Check if we nedd display full tax total info
     *
     * @return bool
     */
    public function displayFullSummary()
    {
        return true;
    }

    /**
     * Get data (totals) source model
     *
     * @return \Magento\Framework\DataObject
     */
    public function getSource()
    {
        return $this->_source;
    } 
    public function getStore()
    {
        return $this->_order->getStore();
    }

      /**
     * @return Order
     */
    public function getOrder()
    {
        return $this->_order;
    }

    /**
     * @return array
     */
    public function getLabelProperties()
    {
        return $this->getParentBlock()->getLabelProperties();
    }

    /**
     * @return array
     */
    public function getValueProperties()
    {
        return $this->getParentBlock()->getValueProperties();
    }

    /**
     * Initialize all order totals relates with tax
     *
     * @return \Magento\Tax\Block\Sales\Order\Tax
     */
     public function initTotals()
    {
        $parent = $this->getParentBlock();
        if($parent){
            $this->_order = $parent->getOrder();
            $this->_source = $parent->getSource();
            $Tax = $this->_source->getTaxAmount();

            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
            
            $storeId = $storeManager->getStore()->getStoreId();
            $websiteId = $storeManager->getWebsite()->getWebsiteId();
            $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
            $scopeId = $storeId ? $storeId : $websiteId;
            if($scopeId!=$storeId){
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
            }
            if($scopeId){
                $debug_mod =  $this->_scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $scope,$scopeId);
            }
            else{
                $debug_mod =  $this->_scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $scope);
            }

            $log = new LogTracking($this->_scopeConfig); 
            if($debug_mod){
                $log->debug_mode_on($debug_mod,
                    'block/Tax.php going inside Tax-'.$Tax,
                    null, null ,$Tax);
            }

            if ($Tax > 0){
            $_tax = new \Magento\Framework\DataObject(
                [
                    'code' => 'tax',
                    'strong' => false,
                    'value' =>  $this->_source->getTaxAmount(),
                    'label' => __('Estimated Tax'),
                ]
            );
            $parent->addTotal($_tax, 'tax');
        }
    }
        return $this;
    }
    /**
     * @return array
     */
    public function getsitekeys()
    {
        return  $this->_checkoutSession->getSiteKey()['site_key_secret'];
    }

    /**
     * @return array
     */
    public function getSiteKeysAPI(){
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        
        $storeId = $storeManager->getStore()->getStoreId();
        $websiteId = $storeManager->getWebsite()->getWebsiteId();
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $scopeId = $storeId ? $storeId : $websiteId;
            if($scopeId!=$storeId){
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
            }
        if($scopeId){
            $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $scope,$scopeId);
            $quiversSandbox = $this->_scopeConfig->getValue("quivers/parameters/quivers_sanndbox", $scope,$scopeId);
            $ref_id = $this->_scopeConfig->getValue("quivers/parameters/merchant_ref_id", $Scope,$scopeId);
        }
        else{
            $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $scope);
            $quiversSandbox = $this->_scopeConfig->getValue("quivers/parameters/quivers_sanndbox", $scope);
            $ref_id = $this->_scopeConfig->getValue("quivers/parameters/merchant_ref_id", $scope);
        }

        $log = new LogTracking($scopeConfig);
        $checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
        $quote = $checkout_session->getQuote();
        if($quiversSandbox == 0) {
            $middlewareurl ='https://middleware.quivers.com/profile/';
        }
        else {
            $middlewareurl ='https://middleware.quiversdemo.com/profile/';
        }
        $this->curlClient->setOption(CURLOPT_CONNECTTIMEOUT, 9);
        $this->curlClient->setOption(CURLOPT_TIMEOUT, 10);
        $sitedata = [
            "client_type"=> "Magento2",
            "business_refid"=> $ref_id
        ];
          
        try {
            $this->curlClient->post($middlewareurl.'retrieve/sitekeys/',json_encode( $sitedata));
            $response = $this->curlClient->getBody();
            $this->_checkoutSession->setSiteKey(json_decode($response,true));
            if($debug_mod){
                $log->debug_mode_on($debug_mod,
                    'Site key APIs in "Tax" ',
                    null,  $response,$sitedata);
            }
        
        } catch (\Exception $e) {
            $log->debug_mode_on($debug_mod,
                    'Catch  "Tax.php" '.$quote->getBillingAddress()->getEmail(),
                    null,  null,$e->getMessage());
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().'Site key APIs not working please check again.'));
        }
    }

}

