<?php
namespace Quivers\Integration;

use \Magento\Framework\HTTP\Client\Curl as CurlClient;

class LogTracking
{

	protected $logEndpoint = 'logs/';
	protected $curlClient;
	protected $scopeConfig;
	
	const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
	const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
	
	public function __construct(\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig)
    {
        $this->curlClient = new CurlClient();
        $this->scopeConfig = $scopeConfig;
    }

    public function session_start($order_detail, $time){
        $logData = [
            "order_detail" => $order_detail,
            "time" => $time
        ];
         $this->logging("SESSION START", $logData);
    }

    public function validate_api_call($request, $response, $start_time, $end_time){
        $logData = [
            "start_time" => $start_time,
            "end_time" => $end_time,
            "request" => $request,
            "response" => $response
        ];
        $this->logging("VALIDATE API CALL", $logData);
    }

    public function countries_api_call($request, $response, $start_time, $end_time){
     $logData = [
            "request" => $request,
            "response" => $response,
            "start_time" => $start_time,
            "end_time" => $end_time
        ];
     $this->logging("COUNTRIES API CALL", $logData);
    }

    public function final_payment_intent($payment_method, $time, $response){
     $logData = [
            "payment_method" => $payment_method,
            "time" => $time,
            "response" => $response
        ];
     $this->logging("PAYMENT INTENT", $logData);
    }
    
    public function update_intent($request, $response, $start_time, $end_time){
     $logData = [
            "request" => $request,
            "response" => $response,
            "start_time" => $start_time,
            "end_time" => $end_time
        ];
     $this->logging("PAYMENT INTENT UPDATED", $logData);
    }

    public function captcha_data($customerEmail,$token ,$captcha_response){
        $logData = [
                "customer mail"=>$customerEmail,
                "token" => $token,
                "captcha_response" => $captcha_response
            ];
        $this->logging("CAPTCHA RESPONSE", $logData);
    }

    public function payment_request($order_data, $payment_method, $time, $response, $success){
     $logData = [
            "order_data" => $order_data,
            "payment_method" => $payment_method,
            "time" => $time,
            "response" => $response,
            "success" => $success
        ];
        
     $this->logging("PAYMENT REQUEST", $logData);
    }

    public function print_statement($message){
     $logData = [
            "message" => $message
        ];
     $this->logging("Message", $logData);
    }

    public function session_end($order_detail, $time){
     $logData = [
            "order_detail" => $order_detail,
            "time" => $time
        ];
     $this->logging("SESSION END", $logData);
    }

    public function debug_mode_on($debug_mode,$function_name, $request_data, $response ,$data){
        $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $storeId = $storeManager->getStore()->getStoreId();
        $websiteId = $storeManager->getWebsite()->getWebsiteId();
        $scopconfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $scopeId = $storeId ? $storeId : $websiteId;
        if($scopeId!=$storeId){
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        }
        if($scopeId){
            $ref_id= $scopconfig->getValue("quivers/parameters/merchant_ref_id", $scope,$scopeId);
        }
        else{
            $ref_id= $scopconfig->getValue("quivers/parameters/merchant_ref_id", $scope);
        }
        $logData = [
               "debug_mode" => $debug_mode,
               "function_name" => $function_name,
               "request" => $request_data,
               "response" => $response,
               "data" => $data
           ];
        $this->logging("DEBUG MODE - Magento2 -> ".$ref_id." ".$function_name, $logData);
       }

   	public function logging($type, $data)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        
        $storeId = $storeManager->getStore()->getStoreId();
        $websiteId = $storeManager->getWebsite()->getWebsiteId();
        $scopeId = $storeId ? $storeId : $websiteId;
        if($scopeId!=$storeId){
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        }
        if($scopeId){
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $scope,$scopeId);
            $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $scope,$scopeId);
        }
        else{
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $scope);
            $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $scope);
        }
        $baseUrl = 'https://middleware.quiversdemo.com/profile/';
        if($quiversSandbox == 0) {
            $baseUrl = 'https://middleware.quivers.com/profile/';
        }
    	$logdata = [
    		"api_key" => "apiKey ".$quiversAPIKey,
    		"uuid" => uniqid(),
            "type" => $type,
    		"log_data" => $data
    	];
    	$logUrl = $baseUrl.$this->logEndpoint;  
        $this->curlClient->post($logUrl, json_encode($logdata));
    }
}