<?php 

namespace Quivers\Integration\Model;
use Magento\Payment\Model\CcGenericConfigProvider;
use Magento\Payment\Model\CcConfig;
use Stripe\Stripe;
use Quivers\Integration\LogTracking;


/**
 * ConfigProvider Class
 */
class ConfigProvider extends CcGenericConfigProvider
{

	const CODE = 'quivers_integration';
	const PATH = 'ecommerce';

    protected $scopeConfig;

	public function __construct(
	    CcConfig $ccConfig,
	    \Magento\Payment\Helper\Data $paymentHelper,
	    \Magento\Checkout\Model\Session $checkoutSession,
	    \Magento\Customer\Model\Session $customerSession,
	    \Magento\Payment\Model\Config $paymentConfig,
	    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
	    \Magento\Framework\Message\ManagerInterface $messageManager,
	    array $methodCodes = []
	) {
	    // $this->paymentHelper    = $paymentHelper;
	    // $this->checkoutSession  = $checkoutSession;
	    $this->customerSession  = $customerSession;
	    // $this->paymentConfig    = $paymentConfig;
	    $this->scopeConfig = $scopeConfig;
	    $this->messageManager = $messageManager;
	    parent::__construct($ccConfig, $paymentHelper, [static::CODE]);
	}

	/**
	 * Get checkout config.
	 *
	 * @return array
	 */
	public function getConfig()
	{	
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        
        $storeId = $storeManager->getStore()->getStoreId();
        $websiteId = $storeManager->getWebsite()->getWebsiteId();
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);

		$scopeId = $storeId ? $storeId : $websiteId;
        if($scopeId!=$storeId){
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        }
		if($scopeId){
			$publicKey = $this->scopeConfig->getValue('payment/quivers_integration/public_key', $scope,$scopeId);
			$title = $this->scopeConfig->getValue('payment/quivers_integration/title', $scope,$scopeId);
			$payPalTitle = $this->scopeConfig->getValue('payment/quivers_integration_paypal/title', $scope,$scopeId);
			$payPalClientId = $this->scopeConfig->getValue('payment/quivers_integration_paypal/client_id', $scope,$scopeId);
			$payPalSandbox = $this->scopeConfig->getValue('payment/quivers_integration_paypal/quivers_paypal_sanndbox', $scope,$scopeId);
			$active = $this->scopeConfig->getValue('quivers/parameters/active', $scope,$scopeId);
			$quiversSandbox = $this->scopeConfig->getValue('quivers/parameters/quivers_sanndbox', $scope,$scopeId);
			$debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $scope,$scopeId);
        }
        else{
            $publicKey = $this->scopeConfig->getValue('payment/quivers_integration/public_key', $scope);
			$title = $this->scopeConfig->getValue('payment/quivers_integration/title', $scope);
			$payPalTitle = $this->scopeConfig->getValue('payment/quivers_integration_paypal/title', $scope);
			$payPalClientId = $this->scopeConfig->getValue('payment/quivers_integration_paypal/client_id', $scope);
			$payPalSandbox = $this->scopeConfig->getValue('payment/quivers_integration_paypal/quivers_paypal_sanndbox', $scope);
			$active = $this->scopeConfig->getValue('quivers/parameters/active', $scope);
			$quiversSandbox = $this->scopeConfig->getValue('quivers/parameters/quivers_sanndbox', $scope);
			$debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $scope);    
        }

		$customer = $this->customerSession->getCustomer();
		$log = new LogTracking($this->scopeConfig);
		$email =$this->customerSession->getCustomer()->getEmail();
		$checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
        $quote = $checkout_session->getQuote();
		//debug logs
		
		if($debug_mod){
			$log->debug_mode_on($debug_mod,'customer- '.$email,null,null,$customer->getData());
		}
		$customerId = $customer->getId();
		$stripeCustomerId = $customer->getQuiversStripeCustomerId();
		//debug logs
		if($debug_mod){
			$log->debug_mode_on($debug_mod,'stripeCustomerId- '.$email,null,null,
			["customerId"=>$customer->getId(),
			"stripeCustomerID" => $stripeCustomerId]);
		}
		try{
			$customersList = \Stripe\Customer::all();
			foreach($customersList as $customerKey){
				$livemode = $customerKey->livemode;
				if($quiversSandbox == 1 && $livemode == true)
				{
					unset($customersList);
				}
			}
		}catch (\Exception $e) {
			$log->debug_mode_on($debug_mod,
                    'Catch  "includedTax.php" '.$quote->getBillingAddress()->getEmail(),
                    null,  null,$e->getMessage());
			$customersList = array();
		}

		try{
			if(empty($customersList)){
				$stripeCustomerId = '';
			}
			$stripeCards = [];
			$selectedCard = [];
			$paymentmethods =[];
			//debug logs
			if($debug_mod){
				$log->debug_mode_on($debug_mod,'customersList_check- '.$email,null,null,
				[
				"stripeCustomerId-2" => $stripeCustomerId]);
			}
			
			if($stripeCustomerId) {

				$allpaymentmethod=\Stripe\PaymentMethod::all([
					'customer' => $stripeCustomerId,
					'type' => 'card'
				]);
				foreach($allpaymentmethod->data as $paymentmethod) 
				{	
						$paymentmethod_detail=[
							'source_id'=>$paymentmethod->id,
							'exp_month'=>$paymentmethod->card->exp_month,
							'exp_year'=>$paymentmethod->card->exp_year,
							'last4'=>$paymentmethod->card->last4,
							'funding'=>$paymentmethod->card->funding,
							'country'=>$paymentmethod->card->country
						];
						$paymentmethods[]=$paymentmethod_detail;
				}

				$stripeCustomer = \Stripe\Customer::retrieve($stripeCustomerId);
				if($stripeCustomer) {
					$savedCardIds = array_map("trim", explode(',', $customer->getQuiversStripeCardIds() ?? ''));
					if($stripeCustomer->id===$stripeCustomerId && $stripeCustomer->sources) {
						$cards = $stripeCustomer->sources->data;
						$card=[];
						if(count($cards)>0) {
							foreach($cards as $card) {
								$passed=false;
								if(isset($card->card) && in_array($card->id, $savedCardIds)) {
									$details = [
										'paymentSourceId'=>$card->id,
										'last4' => $card->card->last4,
										'expMonth' => $card->card->exp_month,
										'expYear' => $card->card->exp_year,
										'country' => $card->card->country,
										'cardType' => $card->card->funding,
										'label' => "XXXX-XXXX-XXXX-".$card->card->last4.' '.$card->card->exp_month.'/'.$card->card->exp_year,
									];
									//debug logs
									if($debug_mod){
										$log->debug_mode_on($debug_mod,'customersdetails- '.$email,null,null,$details);
									}
									$stripeCards[] = $details;
										if($card->id===$stripeCustomer->default_source) {
											$selectedCard = $details;
										}
								}
							}
						}
					}
				}
			}
			else{
				$card=[];
			} 
			$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
			$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
			$baseUrl=$storeManager->getStore()->getBaseUrl();
			if(!isset($baseUrl)){
				throw new \Magento\Framework\Validator\Exception(__());
			}
			$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
			$customer_session=$objectManager->get('\Magento\Customer\Model\Session');
			$config = parent::getConfig();
			$config = array_merge_recursive($config, [
				'validation' => $active ,
				'payment' => [
					static::CODE => [
						'dubug_mode'=>$debug_mod,
						'publicKey' => $publicKey,
						'siteKey' =>isset($customer_session->getSiteKey()['site_key'])?$customer_session->getSiteKey()['site_key']:0,
						'title'=> $title,
						'payPalTitle' => $payPalTitle,
						'payPalClientId' => $payPalClientId,
						'payPalSandbox' => $payPalSandbox,
						'stripeCards' => $stripeCards,
						'selectedCard' => $selectedCard,
						'customerId' => $customerId,
						'baseUrl' => $baseUrl,
						'paymentmethods'=> $card,
						'path'=>static::PATH
					],
				],
			]);
			//debug logs
			if($debug_mod){
				$log->debug_mode_on($debug_mod,'config- CONFIG-PROVIDER- '.$email,null,null,$config);
			}
			return $config;
		}
		catch (\Exception $e) {
			$log->debug_mode_on($debug_mod,
			'Catch  "ConfigProvider.php -" Sorry! Looks like the Payment Method is not yet available to use because it has not yet been Fully Configured. '.$quote->getBillingAddress()->getEmail(),
			null,  null,$e->getMessage());
		$this->messageManager->addErrorMessage(__("Sorry! Looks like the Payment Method is not yet available to use because it has not yet been Fully Configured."));
        throw new \Magento\Framework\Validator\Exception(__('Stripe public key and secret keys are Invalid , Get these from your Stripe account API settings.'));}
	}

}
