<?php

namespace Quivers\Integration\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Customer\Model\Session as CustomerSession;

class OrderObserver implements ObserverInterface
{
    protected $logger;

    protected $scopeConfig;

    protected $customerSession;

    public function __construct(
        Logger $logger,
        CustomerSession $customerSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->logger = $logger;
        $this->customerSession = $customerSession;
        $this->scopeConfig = $scopeConfig;
    }

    public function execute(EventObserver $observer)
    {
      $order = $observer->getEvent()->getOrder();

      $method = $order->getPayment()->getMethod();
     
      if( $method == 'quivers_integration'){
        $payment_intentId= $this->customerSession->getMyValue();

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        
        $storeId = $storeManager->getStore()->getStoreId();
        $websiteId = $storeManager->getWebsite()->getWebsiteId();
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $scopeId = $storeId ? $storeId : $websiteId;
        if($scopeId!=$storeId){
            $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        }
        if($scopeId){ 
            $secret_key = $this->scopeConfig->getValue("payment/quivers_integration/secret_key", $scope,$scopeId);
            $newOrderStatus = $this->scopeConfig->getValue("payment/quivers_integration/order_status", $scope,$scopeId);
        }
        else{
            $newOrderStatus = $this->scopeConfig->getValue("payment/quivers_integration/order_status", $scope);
            $secret_key = $this->scopeConfig->getValue("payment/quivers_integration/secret_key", $sScope);
        }

        
        $stripe= new \Stripe\StripeClient($secret_key);
        $payment_intent=$stripe->paymentIntents->retrieve(
              $payment_intentId,
              []
            );
        
        if ($payment_intent["status"]== "requires_action"|| $payment_intent["status"]== "requires_source_action"){     
          if ($newOrderStatus) {
              // Set the order status according to the configuration
              $order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT, true)->save();
              $order->setStatus(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT, true)->save();
              $order->addStatusToHistory($order->getStatus(), 'Payment Source requires Confirmation');
              $order->save();        
          }     
        }

        if ($payment_intent["status"]== "requires_source"){ 
          if ($newOrderStatus) {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); 
            $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
            $order->setState(\Magento\Sales\Model\Order::STATE_CANCELED, true)->save();
            $order->setStatus(\Magento\Sales\Model\Order::STATE_CANCELED, true)->save();
            $order->addStatusToHistory($order->getStatus(),  $customer_session->getPaymentError());
            $order->save();        
          } 
        }     
      }
      if ($method != 'quivers_integration' && $method != 'quivers_integration_paypal')
          return;
    }
}
