define([
    'jquery',
    'Magento_Payment/js/view/payment/cc-form',
    'Stripe',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/checkout-data'
],
function ($, Component, stripe, quote, checkoutdata) {
    'use strict';
    var config=window.checkoutConfig.payment.quivers_integration;

    return Component.extend({
        defaults: {
            template: 'Quivers_Integration/payment/stripe-form',
            publicKey: config ? config.publicKey : '',
            siteKey: config ? config.siteKey : '',
            dubug_mode: config ? config.dubug_mode : 0,
            title: config ? config.title: 'Pay By Card',
            storedCards: config ? config.stripeCards: [],
            selectedCard: config ? config.selectedCard : [],
            customerId: config ? config.customerId : [] ,
            baseUrl: config ? config.baseUrl: '',
            paymentmethods: config ? config.paymentmethods: [],
            path: config ? config.path: '',
            quiversErrorMessage : ". Please reload the page and try again. If the issue still persists please contact Quivers at 805-250-4555."
        },

        initialize: function () {
            var self = this;

            this._super();

            // Initialize Stripe object thinger
            this.stripe = Stripe(this.publicKey);
            this.elements = this.stripe.elements();
        },

        onCardElementRender: function()
        {   
            var me =this;
            var sitekey = this.siteKey?this.siteKey:'SITE_KEY_NOT_THERE';
   
            // var dubug_mode = this.dubug_mode?this.dubug_mode:'OFF';
        
        
            if(this.dubug_mode){
                console.log("DEBUG MODE :" + this.dubug_mode);
                console.log("quote=" +JSON.stringify(quote.getQuoteId()));
                  console.log("shipping address=" +JSON.stringify(quote.shippingAddress()));
                console.log("customer id ="+this.customerId);
                console.log("billing address ="+JSON.stringify( quote.billingAddress()));
                
            }
      
            var style = {
            base: {
                // Add your base input styles here. For example:
                fontSize: '16px',
                color: "#32325d",
              }
            };

            // Create an instance of the card Element.
            this.card = this.elements.create('card', {style: style, hidePostalCode: true});
            // Add an instance of the card Element into the `card-element` <div>.
            this.card.mount('#card-element');
            var button = document.getElementById("action primary checkout button");
            button.disabled = false;
            this.card.addEventListener('change', function(event) {
                var displayError = document.getElementById('card-errors');
                if (event.error) {
                    displayError.textContent = event.error.message;
                    button.disabled = true;
                } else {
                    button.disabled = false;
                    displayError.textContent = '';
                }
            });
            grecaptcha.ready(function() {
                if(this.dubug_mode){
                    console.log('Inside Captcha ready');
                }
                grecaptcha.render("captcha_box", {
                    "sitekey": sitekey,
                    "callback" :function() {
                        var response = grecaptcha.getResponse();
                        if(response.length != 0){
                            if(this.dubug_mode){
                                console.log('Submit clicked in case of no response of captcha');
                            }
                            jQuery('#form-submit').trigger('click');
                        }else{
                            if(this.dubug_mode){
                                console.log('captcha execute');
                            }
                            grecaptcha.execute();
                        }
                    },
                    "size" : 'invisible'
                });
            });
        },

        context: function() {
            return this;
        },

        getCode: function() {
            return 'quivers_integration';
        },

        isActive: function() {
            return true;
        },
        
        isPlaceOrderActionAllowed: function() {
            return true;
        },
      handleStripeToken: function () {
            if(this.dubug_mode){
                console.log('handle stripe Token Function ');
            }
            
            this.placeOrder()
        },

        handleStoredCard: function () 
        {    
            var me =this;
           if(this.dubug_mode){
                console.log('inside handleStoredCard Function');
            }
            var address = checkoutdata.getNewCustomerBillingAddress();
            if (address == null) {
                address = quote.shippingAddress();
            }       
            me.handleStripeToken();
        },

        handle3dStripe: function () 
        {    
            var me =this;
           if(this.dubug_mode){
                console.log('inside handle3dStripe Function');
            }
            var address = checkoutdata.getNewCustomerBillingAddress();
            if (address == null) {
                address = quote.shippingAddress();
            }

            var name = address.firstname+" "+address.lastname;
            
            var postcode = (address.postcode);
            var payment_method_code = this.getCode();
                    
            globalThis.payment_source_id=null;
            let element = document.body;
            me.stripe.createPaymentMethod({
                type: 'card',
                card: this.card,
                billing_details: {
                  name: name,
                },
              })
              .then(function(result) {
                    if(me.dubug_mode){
                        console.log("Payment Method created is "+JSON.stringify(result.paymentMethod.id));
                    }         
                    if (result.error) 
                    {
                        console.log("Process Stop payemnt method throws error "); 
                    }   
                    me.save_payment_method(result);
                    me.stripe.createSource(me.card, {
                                type: 'card'
                        }).then(function(result) {
                            if(result==undefined){
                                console.log("Result is undefined ");
                            }
                            else{
                                globalThis.payment_source_id=result.source.id
                            }
                            me.handleStripeToken();
                        }).catch(function(e){
                            if(me.dubug_mode){
                                console.log("Error- Source ID not created");
                                console.log(e.message);
                            }
                            var errorElement = document.getElementById('card-errors');
                            var error = "Error processing - Source ID  not created. If the issue persists,please contact Quivers at 805-250-4555."
                            errorElement.textContent = error;
                            element.dispatchEvent(new Event("processStop"));
                        }); 
                
              }).catch(function(e) {
                    if(me.dubug_mode){
                        console.log("Error- Payment Method not created");
                        console.log(e.message); 
                    }
                    var errorElement = document.getElementById('card-errors');
                    var error = "Error processing - Payment Method not created. If the issue persists,please contact Quivers at 805-250-4555."
                    errorElement.textContent = error;
                    element.dispatchEvent(new Event("processStop"));
              });  
        },

        save_payment_method: function(result){
            var payment_Method = result;
            if(result)
            {
                globalThis.payment_method = payment_Method;
            }
            else {
                if(globalThis.payment_method==undefined){        
                    return;
                }
               return globalThis.payment_method.paymentMethod.id;
            }
           
        },

        getToken: function(value) {
            var payment_method_code = this.getCode();
            var element = $('#' + payment_method_code + '-token');
            if(this.dubug_mode){
                console.log("GET TOKEN FUNCTION");
                console.log(element.val());
            }    
            return element.val();
        },

        setToken: function(value) {
            if(this.dubug_mode){
                console.log("SET TOKEN FUNCTION");
                console.log(value);
            }    
            var payment_method_code = this.getCode();
            var element = $('#' + payment_method_code + '-token');
            element.val(value);
        },

        placeOrder: function() {
            this._super();
            if(this.dubug_mode){
                console.log("In place order ");
            }      
            var address = checkoutdata.getNewCustomerBillingAddress();
            if (address == null) {
                address = quote.shippingAddress();
            }
            var postcode = (address.postcode);
            var payment_method_code = this.getCode();
        },

        onPlaceOrderClick: function() {
            if(this.dubug_mode){
              console.log("In onPlaceOrderClick Function ");
            }
            let element = document.body;
            element.dispatchEvent(new Event("processStart"));
            if(this.dubug_mode){
                console.log("Process Start ");
               
            }
            var me =this;
            
            var paymentSourceId = jQuery('#'+this.getCode()+'-card-id').children(':selected').attr('value');
            if(this.dubug_mode){
                console.log("paymentSourceId ",paymentSourceId);
            }
         
            var address = checkoutdata.getNewCustomerBillingAddress();

            if (address == null) {
                    address = quote.shippingAddress();
                  
            }
 
            if(address ==null){
                var errorElement = document.getElementById('card-errors');
                var error = "No Shipping and billing address present in Quote "+this.quiversErrorMessage;
                errorElement.textContent = error;
                element.dispatchEvent(new Event("processStop"));
                var button = document.getElementById("action primary checkout button");
                button.disabled = true;
                if(this.dubug_mode){
                    console.log("Process Stop ");
                }
                return;
            }else{
                var errorElement = document.getElementById('card-errors');
                if(!address.postcode || !address.city) {
                    var error = "Missing address details in Quote "+this.quiversErrorMessage;
                    errorElement.textContent = error;
                    element.dispatchEvent(new Event("processStop"));
                    var button = document.getElementById("action primary checkout button");
                    button.disabled = true;
                    if(this.dubug_mode){
                        console.log("Process Stop "); 
                    }
                    return;
                }   
               
            }
       
            if(this.dubug_mode){
                console.log("address billing ",checkoutdata.getNewCustomerBillingAddress());
                console.log("address quote  shipping ",quote.shippingAddress());
            }
            if (checkoutdata.getNewCustomerBillingAddress() == null) {
                address = quote.shippingAddress();
        }
            if(paymentSourceId){
                if(this.dubug_mode){
                    console.log("Payment Source Id Exist");
                }
                this.setToken(paymentSourceId);
                var data=this.getSelectedCardFromList()[0].childNodes[0].data.split(" ");
                if(this.dubug_mode){
                    console.log("After setting token ");
                    console.log("data", data);
                }

                globalThis.payment_source_id=paymentSourceId;


                var last4=data[0].split("-")[3];
                var exp_month=data[1].split("/")[0];
                var exp_year=data[1].split("/")[1];
                for(var i = 0 ; i < this.paymentmethods.length; i++) {
                    if(this.paymentmethods[i]["exp_month"]==exp_month && this.paymentmethods[i]["exp_year"]==exp_year && 
                        this.paymentmethods[i]["last4"]==last4){

                        var payment_method = this.paymentmethods[i]["payment_method_id"];
                        break;
                    }
                }
             
                if(this.dubug_mode){
                    console.log("Try to create Payment intent by calling the controller");
                    console.log(this.baseUrl);
                    console.log(this.path);
                }

                $.get(this.baseUrl+this.path, {},function(response)
                {
                    if(this.dubug_mode){
                        console.log("Response to Create Payment Intent",response);
                    }
                    if (!response || !response.client_secret) {

                        if(this.dubug_mode){
                            console.log("client_secret not found or response not found"); 
                        }
                        let element = document.body;
                        element.dispatchEvent(new Event("processStop"));
                        if(this.dubug_mode){
                            console.log("Process Stop ");
                           
                        }
                        me.setToken(null);
                        var errorElement = document.getElementById('card-errors');
                        var error = "Error processing your payment due to incorrect Stripe keys. Please check your payment method settings and try again. If the issue persists, If the issue persists please contact Quivers at 805-250-4555."
                        errorElement.textContent = error;
                        setTimeout(function(){window.location.reload(true);}, 5000);
                        return;
                    }

                    var data = {
                        payment_method: payment_method
                    };
                    if(this.dubug_mode){
                        console.log("Payload for confirmCardPayment",data);
                    }
                    me.handleStoredCard();
                });
            }
            else{
                var address = checkoutdata.getNewCustomerBillingAddress();
                if (address == null) {
                    address = quote.shippingAddress();
                }
                var postcode = (address.postcode);
                var payment_method_code = this.getCode();
                var data = {
                    payment_method: {
                        card: this.card,
                        billing_details: {
                            address :{
                                postal_code: postcode
                            }
                          },
                    }   
                };
                if(this.dubug_mode){
                    console.log("NO PAYMENT SOURCE ID: Payload for confirmCardPayment ", data);
                    console.log("base url:"+this.baseUrl+this.path);
                }
                
                $.get(this.baseUrl+this.path, {},function(response){
                   if(this.dubug_mode){
                        console.log("Response to Create Payment Intent 2",response);
                    }
                    if(response.client_secret=='email'){
                        let element = document.body;
                        element.dispatchEvent(new Event("processStop"));      
                        console.log("Process Stop ");    
                        me.setToken(null);
                        var errorElement = document.getElementById('card-errors');
                        var error = "Error please enter the mail id."
                        errorElement.textContent = error;
                        setTimeout(function(){window.location.reload(true);}, 5000);
                        return;
                    }
                    if (!response || !response.client_secret) {
                        let element = document.body;
                        element.dispatchEvent(new Event("processStop"));
                        console.log("Process Stop ");
                        me.setToken(null);
                        var errorElement = document.getElementById('card-errors');
                        var error = "Error processing Payments by Card.\
                        Please choose a different payment method and try again.\
                        If the issue persists please contact Quivers at 805-250-4555."
                        errorElement.textContent = error;
                        setTimeout(function(){window.location.reload(true);}, 5000);
                        return;
                    }
                   me.handle3dStripe();                 
                });
            }
        },

        getcaptcha: function () {
            if(this.dubug_mode){
                console.log("In getcaptcha execute Function ");
            }
            grecaptcha.execute().then(response => {
                jQuery('#form-submit').trigger('click');
            }).catch(e => {
                if(this.dubug_mode){
                    console.log("Going inside catch of captcha and need to submit");
                }
                 jQuery('#form-submit').trigger('click');
            });
        },

        getTitle: function() {
            return this.title;
        },

        getData: function()
        {
            var data = {
                'method': this.getCode(),
            };
           var cc_save_card = jQuery('#save_my_card_checkbox input').prop('checked');
           var cc_save_card_check =[undefined,null,''];
   
            data.additional_data = {
                   'cc_stripejs_token':globalThis.payment_source_id,
                   'captcha_token': grecaptcha.getResponse()?  grecaptcha.getResponse():null,
                   'cc_save_card': !cc_save_card_check.includes(cc_save_card)? cc_save_card:false,
                   'payment_method': this.save_payment_method()
                };
            if(this.dubug_mode){
                console.log("In getData Function with data");
                console.log(JSON.stringify(data));
            }
            return data;
        },

        getStoredCards: function() {
            return this.storedCards;
        },

        getSelectedCard: function() {
            return this.selectedCard.label;
        },

        getCardSelectVisibility: function() {
            return this.storedCards.length
        },

        getSelectedCardFromList: function() {
            var selected = jQuery('#'+this.getCode()+'-card-id').find(":selected");
            return selected;
        },

        getFormElementVisibility: function() {
            var selected = this.getSelectedCardFromList();
            if(!this.getCardSelectVisibility() || !selected.attr('value')) return 1;
            else return 0;
        },

        getSaveCardElementVisibility: function() {
            if(!this.customerId) return 0;
            return this.getFormElementVisibility();
        },

        selectCardAfterRender: function() {
            //handle dropdown changes in card element
            var self = this;
            var cardElement = jQuery('#'+this.getCode()+'-card-id');
            cardElement.on('change', function() {
                if(cardElement.children(':selected').attr('value')) {
                    jQuery('#'+self.getCode()+'-form').css('display', 'none');
                    jQuery('#save_my_card_checkbox').css('display', 'none');
                }
                else {
                    jQuery('#'+self.getCode()+'-form').css('display', 'block');
                    jQuery('#save_my_card_checkbox').css('display', 'block');                        
                }
            });
        },
    });
}
);
